# theia
## What is it?
Theia is an effort to abstract a lot of repetitive code that we write in different our platfrom teams into one single generic library. It is also meant to be a single point of control for pulumi versioning and features to prevent breakages based on pulumi. It is written with pulumi automation hence giving users the freedom from a pulumi restricted "stack file" structure. 

## How to use it?
Download the tar.gz file from the latest release artifacts and run `pip install theia-0.1.0rc1.tar.gz` to install the pip package

Now theia library should be available in your python environment. Refer to the examples folder in this repository to understand how theia can be used

## Basic Structure
A very basic code structure needs to be followed when using theia. You have to construct a function with a specific signature of `Callable[(), None]` for theia to work since this signature is expected by the pulumi automation API.
```
from theia import Theia

def infra() -> None:
    # Define Resources
    pass


t = Theia(
    stack="theia-cloudport-stack",
    project="theia-draft",
    program=infra,
    region="us-east-2",
)
t.preview()
```

## Setting up environment for local testing
Install Poetry in the local system

### Works for both mac and linux system
```
curl -sSL https://install.python-poetry.org | python3 -
```

### Setting up poetry project

You can perform testing on already present theia repo that you cloned or you can create your own new project for standalone testing

#### Using theia repo

```
#clone theia repo
git clone https://github.com/amagimedia/theia.git

#Navigate the theia folder
cd theia

#Setting the venv config
poetry config virtualenvs.in-project true

#run poetry shell to login into the venv
poetry shell

#install dependencies packages present inside the pyproject.toml
poetry install
```
#### Creating new project

Create new project using poetry

```
poetry new <project_name>

#navigate to folder
cd <project_name>

#run poetry shell to login into the venv
poetry shell
```

Once the poetry project setup is completed we need to set up poetry to access the amagi pipy server repo

```
poetry source add --priority=explicit amagi http://pypi.iota.amagi.tv

poetry config http-basic.amagi <username> <password>

poetry add theia --source amagi
```

This will install latest version of theia in the Virtual Environment.

#### Note:- Please get the username and password fro platform team for the amagi pipy server repo

### Testing script

When ever you run the poetry shell it will provide you with some virtual enviornment path which looks something like this

```
/Users/<username>/Library/Caches/pypoetry/virtualenvs/<poetry_project>/bin/activate
```

navigate to the path from your local machine not virtaul environemnt to the location mentined below
```
cd /Users/<username>/Library/Caches/pypoetry/virtualenvs/<poetry_project>/lib/<your_python_dir>/site-packages/theia/
```
once you naviagte to this folder add you new theia python file for resource as the dir structure as you need after that you can import the resource as package in you module script and use it.
